# -*- coding: cp1252 -*-

# GeerosInterfaceIMU_3D est bas sur sf9domahrs par Doug Weibel et Jose Julio:
# http://code.google.com/p/sf9domahrs/

# sf9domahrs est bas sur ArduIMU v1.5 par Jordi Munoz, William Premerlani, Jose
# Julio and Doug Weibel:
# http://code.google.com/p/ardu-imu/

# InterfaceIMU_3D est un logiciel libre ; vous pouvez la redistribuer ou
# la modifier suivant les termes de la Licence Gnrale Publique Limite
# GNU telle que publie par la Free Software Foundation ; soit la
# version 3 de la License, soit ( votre gr) toute version ultrieure.
# Ce script est distribu dans lespoir quil sera utile, mais
# SANS AUCUNE GARANTIE : sans mme la garantie implicite de
# COMMERCIALISABILIT ou dADQUATION  UN OBJECTIF PARTICULIER. Consultez
# la Licence Gnrale Publique Limite pour plus de dtails.
# Vous devriez avoir reu une copie de la Licence Gnrale Publique Limite
# GNU avec ce scriptque ; si ce nest pas le cas, crivez  la :
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301, USA.

# Version 2.0 du 17/10/2013 dveloppe par 3Sigma pour la visualisation
# de l'orientation dans l'espace de l'IMU du robot Geeros
# (http://www.geeros.com))
################################################################################

# Ce script ncessite Python 2.7 et VPython 6.05 (http://www.vpython.org/) 

from visual import *
import serial
import string
import math
import socket

from time import time

grad2rad = 3.141592/180.0

# Adresse IP et port de connexion
HOST = '192.168.0.199'
PORT = 2001

def linesplit(socket):
    done = False

    try:
        buff = socket.recv(1024)
        erreur_buff = False
    except:
        erreur_buff = True
        done = True
        pass

    while not done:
        if "\n" in buff:
            (line, buff) = buff.split("\n", 1)
            yield line + "\n"
        else:

            try:
                more = socket.recv(1024)
            except:
                break
            if not more:
                done = True
            else:
                buff = buff + more

    if not erreur_buff:
        if buff:
            yield buff


# Ouverture du socket
s = socket.create_connection((HOST,PORT))
s.setblocking(0)

# Scne principale
scene=display(title="Geeros IMU")
scene.range=(1.2,1.2,1.2)
scene.forward = (1,0,-0.25)
scene.up=(0,0,1)

# Seconde scne (Roll, Pitch, Yaw)
scene2 = display(title='Geeros IMU',x=0, y=0, width=500, height=200,center=(0,0,0), background=(0,0,0))
scene2.range=(1,1,1)
scene.width=500
scene.y=200

scene2.select()
# Roll, Pitch, Yaw
cil_roll = cylinder(pos=(-0.4,0,0),axis=(0.2,0,0),radius=0.01,color=color.red)
cil_roll2 = cylinder(pos=(-0.4,0,0),axis=(-0.2,0,0),radius=0.01,color=color.red)
cil_pitch = cylinder(pos=(0.1,0,0),axis=(0.2,0,0),radius=0.01,color=color.green)
cil_pitch2 = cylinder(pos=(0.1,0,0),axis=(-0.2,0,0),radius=0.01,color=color.green)
arrow_course = arrow(pos=(0.6,0,0),color=color.cyan,axis=(-0.2,0,0), shaftwidth=0.02, fixedwidth=1)

# Labels
label(pos=(-0.4,0.3,0),text="Roulis",box=0,opacity=0)
label(pos=(0.1,0.3,0),text="Tangage",box=0,opacity=0)
label(pos=(0.5,0.3,0),text="Lacet",box=0,opacity=0)
label(pos=(0.6,0.22,0),text="N",box=0,opacity=0,color=color.yellow)
label(pos=(0.6,-0.22,0),text="S",box=0,opacity=0,color=color.yellow)
label(pos=(0.38,0,0),text="O",box=0,opacity=0,color=color.yellow)
label(pos=(0.82,0,0),text="E",box=0,opacity=0,color=color.yellow)
label(pos=(0.75,0.15,0),height=7,text="NE",box=0,color=color.yellow)
label(pos=(0.45,0.15,0),height=7,text="NO",box=0,color=color.yellow)
label(pos=(0.75,-0.15,0),height=7,text="SE",box=0,color=color.yellow)
label(pos=(0.45,-0.15,0),height=7,text="SO",box=0,color=color.yellow)

L1 = label(pos=(-0.4,0.22,0),text="-",box=0,opacity=0)
L2 = label(pos=(0.1,0.22,0),text="-",box=0,opacity=0)
L3 = label(pos=(0.7,0.3,0),text="-",box=0,opacity=0)

# Objets de la scne principale
scene.select()
# Axe de rfrence (x,y,z)
arrow(color=color.green,axis=(1,0,0), shaftwidth=0.02, fixedwidth=1)
arrow(color=color.green,axis=(0,-1,0), shaftwidth=0.02 , fixedwidth=1)
arrow(color=color.green,axis=(0,0,-1), shaftwidth=0.02, fixedwidth=1)
# labels
label(pos=(0,0,0.8),text="Geeros IMU",box=0,opacity=0)
label(pos=(1,0,0),text="X",box=0,opacity=0)
label(pos=(0,-1,0),text="Y",box=0,opacity=0)
label(pos=(0,0,-1),text="Z",box=0,opacity=0)
# Objet IMU
platform = box(length=1, height=0.05, width=1, color=color.blue)
p_line = box(length=1,height=0.08,width=0.1,color=color.yellow)
plat_arrow = arrow(color=color.green,axis=(1,0,0), shaftwidth=0.06, fixedwidth=1)


roll=0
pitch=0
yaw=0
while 1:
    rate(150)
    # Rception des donnes
    sample_string = linesplit(s)
    for line in sample_string:
        if line.find("!ANG:") != -1:          # filtrage des lignes incompltes
            line = line.replace("!ANG:","")   # Supprime "!ANG:"
            words = string.split(line,",")    # Sparation des champs
            if len(words) > 2:
                try:
                    roll = float(words[0])*grad2rad
                    pitch = float(words[1])*grad2rad
                    yaw = float(words[2])*grad2rad
                    axis=(cos(pitch)*cos(yaw),-cos(pitch)*sin(yaw),sin(pitch))
                    up=(sin(roll)*sin(yaw)+cos(roll)*sin(pitch)*cos(yaw),sin(roll)*cos(yaw)-cos(roll)*sin(pitch)*sin(yaw),-cos(roll)*cos(pitch))
                    platform.axis=axis
                    platform.up=up
                    platform.length=1.0
                    platform.width=0.65
                    plat_arrow.axis=axis
                    plat_arrow.up=up
                    plat_arrow.length=0.8
                    p_line.axis=axis
                    p_line.up=up
                    cil_roll.axis=(0.2*cos(roll),0.2*sin(roll),0)
                    cil_roll2.axis=(-0.2*cos(roll),-0.2*sin(roll),0)
                    cil_pitch.axis=(0.2*cos(pitch),0.2*sin(pitch),0)
                    cil_pitch2.axis=(-0.2*cos(pitch),-0.2*sin(pitch),0)
                    arrow_course.axis=(0.2*sin(yaw),0.2*cos(yaw),0)
                    L1.text = str(float(words[0]))
                    L2.text = str(float(words[1]))
                    L3.text = str(float(words[2]))        
                except:
                    #print "Ligne invalide"
                    pass


